# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: proto/queue/v1/queue.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import Dict, List, Optional

import betterproto
from betterproto.grpc.grpclib_server import ServiceBase
import grpclib


@dataclass(eq=False, repr=False)
class QueueSendRequest(betterproto.Message):
    """Request to push a single event to a queue"""

    # The Nitric name for the queue this will automatically be resolved to the
    # provider specific queue identifier.
    queue: str = betterproto.string_field(1)
    # The task to push to the queue
    task: "NitricTask" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class QueueSendResponse(betterproto.Message):
    """Result of pushing a single task to a queue"""

    pass


@dataclass(eq=False, repr=False)
class QueueSendBatchRequest(betterproto.Message):
    # The Nitric name for the queue this will automatically be resolved to the
    # provider specific queue identifier.
    queue: str = betterproto.string_field(1)
    # Array of tasks to push to the queue
    tasks: List["NitricTask"] = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class QueueSendBatchResponse(betterproto.Message):
    """Response for sending a collection of tasks"""

    # A list of tasks that failed to be queued
    failed_tasks: List["FailedTask"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class QueueReceiveRequest(betterproto.Message):
    # The nitric name for the queue this will automatically be resolved to the
    # provider specific queue identifier.
    queue: str = betterproto.string_field(1)
    # The max number of items to pop off the queue, may be capped by provider
    # specific limitations
    depth: int = betterproto.int32_field(2)


@dataclass(eq=False, repr=False)
class QueueReceiveResponse(betterproto.Message):
    # Array of tasks popped off the queue
    tasks: List["NitricTask"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class QueueCompleteRequest(betterproto.Message):
    # The nitric name for the queue  this will automatically be resolved to the
    # provider specific queue identifier.
    queue: str = betterproto.string_field(1)
    # Lease id of the task to be completed
    lease_id: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class QueueCompleteResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class FailedTask(betterproto.Message):
    # The task that failed to be pushed
    task: "NitricTask" = betterproto.message_field(1)
    # A message describing the failure
    message: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class NitricTask(betterproto.Message):
    """A task to be sent or received from a queue."""

    # A unique id for the task
    id: str = betterproto.string_field(1)
    # The lease id unique to the pop request, this must be used to complete,
    # extend the lease or release the task.
    lease_id: str = betterproto.string_field(2)
    # A content hint for the tasks payload
    payload_type: str = betterproto.string_field(3)
    # The payload of the task
    payload: "betterproto_lib_google_protobuf.Struct" = betterproto.message_field(4)


class QueueServiceStub(betterproto.ServiceStub):
    async def send(
        self, *, queue: str = "", task: "NitricTask" = None
    ) -> "QueueSendResponse":

        request = QueueSendRequest()
        request.queue = queue
        if task is not None:
            request.task = task

        return await self._unary_unary(
            "/nitric.queue.v1.QueueService/Send", request, QueueSendResponse
        )

    async def send_batch(
        self, *, queue: str = "", tasks: Optional[List["NitricTask"]] = None
    ) -> "QueueSendBatchResponse":
        tasks = tasks or []

        request = QueueSendBatchRequest()
        request.queue = queue
        if tasks is not None:
            request.tasks = tasks

        return await self._unary_unary(
            "/nitric.queue.v1.QueueService/SendBatch", request, QueueSendBatchResponse
        )

    async def receive(
        self, *, queue: str = "", depth: int = 0
    ) -> "QueueReceiveResponse":

        request = QueueReceiveRequest()
        request.queue = queue
        request.depth = depth

        return await self._unary_unary(
            "/nitric.queue.v1.QueueService/Receive", request, QueueReceiveResponse
        )

    async def complete(
        self, *, queue: str = "", lease_id: str = ""
    ) -> "QueueCompleteResponse":

        request = QueueCompleteRequest()
        request.queue = queue
        request.lease_id = lease_id

        return await self._unary_unary(
            "/nitric.queue.v1.QueueService/Complete", request, QueueCompleteResponse
        )


class QueueServiceBase(ServiceBase):
    async def send(self, queue: str, task: "NitricTask") -> "QueueSendResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def send_batch(
        self, queue: str, tasks: Optional[List["NitricTask"]]
    ) -> "QueueSendBatchResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def receive(self, queue: str, depth: int) -> "QueueReceiveResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def complete(self, queue: str, lease_id: str) -> "QueueCompleteResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_send(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "queue": request.queue,
            "task": request.task,
        }

        response = await self.send(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_send_batch(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "queue": request.queue,
            "tasks": request.tasks,
        }

        response = await self.send_batch(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_receive(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "queue": request.queue,
            "depth": request.depth,
        }

        response = await self.receive(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_complete(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "queue": request.queue,
            "lease_id": request.lease_id,
        }

        response = await self.complete(**request_kwargs)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/nitric.queue.v1.QueueService/Send": grpclib.const.Handler(
                self.__rpc_send,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueueSendRequest,
                QueueSendResponse,
            ),
            "/nitric.queue.v1.QueueService/SendBatch": grpclib.const.Handler(
                self.__rpc_send_batch,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueueSendBatchRequest,
                QueueSendBatchResponse,
            ),
            "/nitric.queue.v1.QueueService/Receive": grpclib.const.Handler(
                self.__rpc_receive,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueueReceiveRequest,
                QueueReceiveResponse,
            ),
            "/nitric.queue.v1.QueueService/Complete": grpclib.const.Handler(
                self.__rpc_complete,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueueCompleteRequest,
                QueueCompleteResponse,
            ),
        }


import betterproto.lib.google.protobuf as betterproto_lib_google_protobuf



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: proto/error/v1/error.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import Dict

import betterproto
from betterproto.grpc.grpclib_server import ServiceBase


@dataclass(eq=False, repr=False)
class ErrorScope(betterproto.Message):
    # The API service invoked, e.g. 'Service.Method'.
    service: str = betterproto.string_field(1)
    # The plugin method invoked, e.g. 'PluginService.Method'.
    plugin: str = betterproto.string_field(2)
    # The plugin method arguments, ensure only non-sensitive data is specified.
    args: Dict[str, str] = betterproto.map_field(
        3, betterproto.TYPE_STRING, betterproto.TYPE_STRING
    )


@dataclass(eq=False, repr=False)
class ErrorDetails(betterproto.Message):
    # The developer error message, explaining the error and ideally solution.
    message: str = betterproto.string_field(1)
    # The error root cause.
    cause: str = betterproto.string_field(2)
    # The scope of the error.
    scope: "ErrorScope" = betterproto.message_field(3)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: proto/faas/v1/faas.proto
# plugin: python-betterproto
import warnings
from dataclasses import dataclass
from typing import AsyncIterable, AsyncIterator, Dict, Iterable, List, Union

import betterproto
from betterproto.grpc.grpclib_server import ServiceBase
import grpclib


@dataclass(eq=False, repr=False)
class ClientMessage(betterproto.Message):
    """Messages the client is able to send to the server"""

    # Client message ID, used to pair requests/responses
    id: str = betterproto.string_field(1)
    # Client initialisation request A worker will not be eligible for triggers
    # until it has identified itself
    init_request: "InitRequest" = betterproto.message_field(2, group="content")
    # Client responsding with result of a trigger
    trigger_response: "TriggerResponse" = betterproto.message_field(3, group="content")


@dataclass(eq=False, repr=False)
class ServerMessage(betterproto.Message):
    """Messages the server is able to send to the client"""

    # Server message ID, used to pair requests/responses
    id: str = betterproto.string_field(1)
    # Server responding with client configuration details to an InitRequest
    init_response: "InitResponse" = betterproto.message_field(2, group="content")
    # Server requesting client to process a trigger
    trigger_request: "TriggerRequest" = betterproto.message_field(3, group="content")


@dataclass(eq=False, repr=False)
class ApiWorker(betterproto.Message):
    api: str = betterproto.string_field(1)
    path: str = betterproto.string_field(2)
    methods: List[str] = betterproto.string_field(3)


@dataclass(eq=False, repr=False)
class SubscriptionWorker(betterproto.Message):
    topic: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class ScheduleWorker(betterproto.Message):
    key: str = betterproto.string_field(1)
    rate: "ScheduleRate" = betterproto.message_field(10, group="cadence")
    cron: "ScheduleCron" = betterproto.message_field(11, group="cadence")


@dataclass(eq=False, repr=False)
class ScheduleRate(betterproto.Message):
    rate: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class ScheduleCron(betterproto.Message):
    cron: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class InitRequest(betterproto.Message):
    """
    InitRequest - Identifies a worker as ready to recieve triggers This message
    will contain information on the type of triggers that a worker is capable
    of handling
    """

    api: "ApiWorker" = betterproto.message_field(10, group="Worker")
    subscription: "SubscriptionWorker" = betterproto.message_field(11, group="Worker")
    schedule: "ScheduleWorker" = betterproto.message_field(12, group="Worker")


@dataclass(eq=False, repr=False)
class InitResponse(betterproto.Message):
    """Placeholder message"""

    pass


@dataclass(eq=False, repr=False)
class TriggerRequest(betterproto.Message):
    """The server has a trigger for the client to handle"""

    # The data in the trigger
    data: bytes = betterproto.bytes_field(1)
    # Should we supply a mime type for the data? Or rely on context?
    mime_type: str = betterproto.string_field(2)
    http: "HttpTriggerContext" = betterproto.message_field(3, group="context")
    topic: "TopicTriggerContext" = betterproto.message_field(4, group="context")


@dataclass(eq=False, repr=False)
class HeaderValue(betterproto.Message):
    value: List[str] = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class QueryValue(betterproto.Message):
    value: List[str] = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class HttpTriggerContext(betterproto.Message):
    # The request method
    method: str = betterproto.string_field(1)
    # The path of the request
    path: str = betterproto.string_field(2)
    # The old request headers (preserving for backwards compatibility) TODO:
    # Remove in 1.0
    headers_old: Dict[str, str] = betterproto.map_field(
        3, betterproto.TYPE_STRING, betterproto.TYPE_STRING
    )
    # The old query params (preserving for backwards compatibility) TODO: Remove
    # in 1.0
    query_params_old: Dict[str, str] = betterproto.map_field(
        4, betterproto.TYPE_STRING, betterproto.TYPE_STRING
    )
    # HTTP request headers
    headers: Dict[str, "HeaderValue"] = betterproto.map_field(
        5, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE
    )
    # HTTP Query params
    query_params: Dict[str, "QueryValue"] = betterproto.map_field(
        6, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE
    )
    # HTTP Path parameters
    path_params: Dict[str, str] = betterproto.map_field(
        7, betterproto.TYPE_STRING, betterproto.TYPE_STRING
    )

    def __post_init__(self) -> None:
        super().__post_init__()
        if self.headers_old:
            warnings.warn(
                "HttpTriggerContext.headers_old is deprecated", DeprecationWarning
            )
        if self.query_params_old:
            warnings.warn(
                "HttpTriggerContext.query_params_old is deprecated", DeprecationWarning
            )


@dataclass(eq=False, repr=False)
class TopicTriggerContext(betterproto.Message):
    # The topic the message was published for
    topic: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class TriggerResponse(betterproto.Message):
    """The worker has successfully processed a trigger"""

    # The data returned in the response
    data: bytes = betterproto.bytes_field(1)
    # response to a http request
    http: "HttpResponseContext" = betterproto.message_field(10, group="context")
    # response to a topic trigger
    topic: "TopicResponseContext" = betterproto.message_field(11, group="context")


@dataclass(eq=False, repr=False)
class HttpResponseContext(betterproto.Message):
    """
    Specific HttpResponse message Note this does not have to be handled by the
    User at all but they will have the option of control If they choose...
    """

    # Old HTTP response headers (deprecated) TODO: Remove in 1.0
    headers_old: Dict[str, str] = betterproto.map_field(
        1, betterproto.TYPE_STRING, betterproto.TYPE_STRING
    )
    # The HTTP status of the request
    status: int = betterproto.int32_field(2)
    # HTTP response headers
    headers: Dict[str, "HeaderValue"] = betterproto.map_field(
        3, betterproto.TYPE_STRING, betterproto.TYPE_MESSAGE
    )

    def __post_init__(self) -> None:
        super().__post_init__()
        if self.headers_old:
            warnings.warn(
                "HttpResponseContext.headers_old is deprecated", DeprecationWarning
            )


@dataclass(eq=False, repr=False)
class TopicResponseContext(betterproto.Message):
    """
    Specific event response message We do not accept responses for events only
    whether or not they were successfully processed
    """

    # Success status of the handled event
    success: bool = betterproto.bool_field(1)


class FaasServiceStub(betterproto.ServiceStub):
    async def trigger_stream(
        self,
        request_iterator: Union[
            AsyncIterable["ClientMessage"], Iterable["ClientMessage"]
        ],
    ) -> AsyncIterator["ServerMessage"]:

        async for response in self._stream_stream(
            "/nitric.faas.v1.FaasService/TriggerStream",
            request_iterator,
            ClientMessage,
            ServerMessage,
        ):
            yield response


class FaasServiceBase(ServiceBase):
    async def trigger_stream(
        self, request_iterator: AsyncIterator["ClientMessage"]
    ) -> AsyncIterator["ServerMessage"]:
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_trigger_stream(self, stream: grpclib.server.Stream) -> None:
        request_kwargs = {"request_iterator": stream.__aiter__()}

        await self._call_rpc_handler_server_stream(
            self.trigger_stream,
            stream,
            request_kwargs,
        )

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/nitric.faas.v1.FaasService/TriggerStream": grpclib.const.Handler(
                self.__rpc_trigger_stream,
                grpclib.const.Cardinality.STREAM_STREAM,
                ClientMessage,
                ServerMessage,
            ),
        }



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: proto/resource/v1/resource.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import Dict, List

import betterproto
from betterproto.grpc.grpclib_server import ServiceBase
import grpclib


class ResourceType(betterproto.Enum):
    Api = 0
    Function = 1
    Bucket = 2
    Queue = 3
    Topic = 4
    Schedule = 5
    Subscription = 6
    Collection = 7
    Policy = 8
    Secret = 9


class Action(betterproto.Enum):
    # Bucket Permissions: 0XX
    BucketFileList = 0
    BucketFileGet = 1
    BucketFilePut = 2
    BucketFileDelete = 3
    # Topic Permissions: 2XX
    TopicList = 200
    TopicDetail = 201
    TopicEventPublish = 202
    # Queue Permissions: 3XX
    QueueSend = 300
    QueueReceive = 301
    QueueList = 302
    QueueDetail = 303
    # Collection Permissions: 4XX
    CollectionDocumentRead = 400
    CollectionDocumentWrite = 401
    CollectionDocumentDelete = 402
    CollectionQuery = 403
    CollectionList = 404
    # Secret Permissions: 5XX
    SecretPut = 500
    SecretAccess = 501


@dataclass(eq=False, repr=False)
class PolicyResource(betterproto.Message):
    principals: List["Resource"] = betterproto.message_field(1)
    actions: List["Action"] = betterproto.enum_field(2)
    resources: List["Resource"] = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class Resource(betterproto.Message):
    type: "ResourceType" = betterproto.enum_field(1)
    name: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class ResourceDeclareRequest(betterproto.Message):
    resource: "Resource" = betterproto.message_field(1)
    policy: "PolicyResource" = betterproto.message_field(10, group="config")
    bucket: "BucketResource" = betterproto.message_field(11, group="config")
    queue: "QueueResource" = betterproto.message_field(12, group="config")
    topic: "TopicResource" = betterproto.message_field(13, group="config")
    collection: "CollectionResource" = betterproto.message_field(14, group="config")
    secret: "SecretResource" = betterproto.message_field(15, group="config")


@dataclass(eq=False, repr=False)
class BucketResource(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class QueueResource(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class TopicResource(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class CollectionResource(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class SecretResource(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class ResourceDeclareResponse(betterproto.Message):
    pass


class ResourceServiceStub(betterproto.ServiceStub):
    async def declare(
        self,
        *,
        resource: "Resource" = None,
        policy: "PolicyResource" = None,
        bucket: "BucketResource" = None,
        queue: "QueueResource" = None,
        topic: "TopicResource" = None,
        collection: "CollectionResource" = None,
        secret: "SecretResource" = None,
    ) -> "ResourceDeclareResponse":

        request = ResourceDeclareRequest()
        if resource is not None:
            request.resource = resource
        if policy is not None:
            request.policy = policy
        if bucket is not None:
            request.bucket = bucket
        if queue is not None:
            request.queue = queue
        if topic is not None:
            request.topic = topic
        if collection is not None:
            request.collection = collection
        if secret is not None:
            request.secret = secret

        return await self._unary_unary(
            "/nitric.resource.v1.ResourceService/Declare",
            request,
            ResourceDeclareResponse,
        )


class ResourceServiceBase(ServiceBase):
    async def declare(
        self,
        resource: "Resource",
        policy: "PolicyResource",
        bucket: "BucketResource",
        queue: "QueueResource",
        topic: "TopicResource",
        collection: "CollectionResource",
        secret: "SecretResource",
    ) -> "ResourceDeclareResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_declare(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "resource": request.resource,
            "policy": request.policy,
            "bucket": request.bucket,
            "queue": request.queue,
            "topic": request.topic,
            "collection": request.collection,
            "secret": request.secret,
        }

        response = await self.declare(**request_kwargs)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/nitric.resource.v1.ResourceService/Declare": grpclib.const.Handler(
                self.__rpc_declare,
                grpclib.const.Cardinality.UNARY_UNARY,
                ResourceDeclareRequest,
                ResourceDeclareResponse,
            ),
        }



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: proto/storage/v1/storage.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import Dict

import betterproto
from betterproto.grpc.grpclib_server import ServiceBase
import grpclib


class StoragePreSignUrlRequestOperation(betterproto.Enum):
    READ = 0
    WRITE = 1


@dataclass(eq=False, repr=False)
class StorageWriteRequest(betterproto.Message):
    """Request to put (create/update) a storage item"""

    # Nitric name of the bucket to store in  this will be automatically resolved
    # to the provider specific bucket identifier.
    bucket_name: str = betterproto.string_field(1)
    # Key to store the item under
    key: str = betterproto.string_field(2)
    # bytes array to store
    body: bytes = betterproto.bytes_field(3)


@dataclass(eq=False, repr=False)
class StorageWriteResponse(betterproto.Message):
    """Result of putting a storage item"""

    pass


@dataclass(eq=False, repr=False)
class StorageReadRequest(betterproto.Message):
    """Request to retrieve a storage item"""

    # Nitric name of the bucket to retrieve from  this will be automatically
    # resolved to the provider specific bucket identifier.
    bucket_name: str = betterproto.string_field(1)
    # Key of item to retrieve
    key: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class StorageReadResponse(betterproto.Message):
    """Returned storage item"""

    # The body bytes of the retrieved storage item
    body: bytes = betterproto.bytes_field(1)


@dataclass(eq=False, repr=False)
class StorageDeleteRequest(betterproto.Message):
    """Request to delete a storage item"""

    # Name of the bucket to delete from
    bucket_name: str = betterproto.string_field(1)
    # Key of item to delete
    key: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class StorageDeleteResponse(betterproto.Message):
    """Result of deleting a storage item"""

    pass


@dataclass(eq=False, repr=False)
class StoragePreSignUrlRequest(betterproto.Message):
    """
    Request to generate a pre-signed URL for a file to perform a specific
    operation, such as read or write.
    """

    # Nitric name of the bucket to retrieve from  this will be automatically
    # resolved to the provider specific bucket identifier.
    bucket_name: str = betterproto.string_field(1)
    # Key of item to generate the signed URL for. The URL and the token it
    # contains will only be valid for operations on this resource specifically.
    key: str = betterproto.string_field(2)
    operation: "StoragePreSignUrlRequestOperation" = betterproto.enum_field(3)
    # Expiry time in seconds for the token included in the signed URL.  Time
    # starts from when the access token is generated, not when this request is
    # made.  e.g. time.Now().Add(expiry * time.Second) on the server
    expiry: int = betterproto.uint32_field(4)


@dataclass(eq=False, repr=False)
class StoragePreSignUrlResponse(betterproto.Message):
    # The pre-signed url, restricted to the operation, resource and expiry time
    # specified in the request.
    url: str = betterproto.string_field(1)


class StorageServiceStub(betterproto.ServiceStub):
    async def read(
        self, *, bucket_name: str = "", key: str = ""
    ) -> "StorageReadResponse":

        request = StorageReadRequest()
        request.bucket_name = bucket_name
        request.key = key

        return await self._unary_unary(
            "/nitric.storage.v1.StorageService/Read", request, StorageReadResponse
        )

    async def write(
        self, *, bucket_name: str = "", key: str = "", body: bytes = b""
    ) -> "StorageWriteResponse":

        request = StorageWriteRequest()
        request.bucket_name = bucket_name
        request.key = key
        request.body = body

        return await self._unary_unary(
            "/nitric.storage.v1.StorageService/Write", request, StorageWriteResponse
        )

    async def delete(
        self, *, bucket_name: str = "", key: str = ""
    ) -> "StorageDeleteResponse":

        request = StorageDeleteRequest()
        request.bucket_name = bucket_name
        request.key = key

        return await self._unary_unary(
            "/nitric.storage.v1.StorageService/Delete", request, StorageDeleteResponse
        )

    async def pre_sign_url(
        self,
        *,
        bucket_name: str = "",
        key: str = "",
        operation: "StoragePreSignUrlRequestOperation" = None,
        expiry: int = 0,
    ) -> "StoragePreSignUrlResponse":

        request = StoragePreSignUrlRequest()
        request.bucket_name = bucket_name
        request.key = key
        request.operation = operation
        request.expiry = expiry

        return await self._unary_unary(
            "/nitric.storage.v1.StorageService/PreSignUrl",
            request,
            StoragePreSignUrlResponse,
        )


class StorageServiceBase(ServiceBase):
    async def read(self, bucket_name: str, key: str) -> "StorageReadResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def write(
        self, bucket_name: str, key: str, body: bytes
    ) -> "StorageWriteResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def delete(self, bucket_name: str, key: str) -> "StorageDeleteResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def pre_sign_url(
        self,
        bucket_name: str,
        key: str,
        operation: "StoragePreSignUrlRequestOperation",
        expiry: int,
    ) -> "StoragePreSignUrlResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_read(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "bucket_name": request.bucket_name,
            "key": request.key,
        }

        response = await self.read(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_write(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "bucket_name": request.bucket_name,
            "key": request.key,
            "body": request.body,
        }

        response = await self.write(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_delete(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "bucket_name": request.bucket_name,
            "key": request.key,
        }

        response = await self.delete(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_pre_sign_url(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "bucket_name": request.bucket_name,
            "key": request.key,
            "operation": request.operation,
            "expiry": request.expiry,
        }

        response = await self.pre_sign_url(**request_kwargs)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/nitric.storage.v1.StorageService/Read": grpclib.const.Handler(
                self.__rpc_read,
                grpclib.const.Cardinality.UNARY_UNARY,
                StorageReadRequest,
                StorageReadResponse,
            ),
            "/nitric.storage.v1.StorageService/Write": grpclib.const.Handler(
                self.__rpc_write,
                grpclib.const.Cardinality.UNARY_UNARY,
                StorageWriteRequest,
                StorageWriteResponse,
            ),
            "/nitric.storage.v1.StorageService/Delete": grpclib.const.Handler(
                self.__rpc_delete,
                grpclib.const.Cardinality.UNARY_UNARY,
                StorageDeleteRequest,
                StorageDeleteResponse,
            ),
            "/nitric.storage.v1.StorageService/PreSignUrl": grpclib.const.Handler(
                self.__rpc_pre_sign_url,
                grpclib.const.Cardinality.UNARY_UNARY,
                StoragePreSignUrlRequest,
                StoragePreSignUrlResponse,
            ),
        }



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: proto/event/v1/event.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import Dict, List

import betterproto
from betterproto.grpc.grpclib_server import ServiceBase
import grpclib


@dataclass(eq=False, repr=False)
class EventPublishRequest(betterproto.Message):
    """Request to publish an event to a topic"""

    # The name of the topic to publish the event to
    topic: str = betterproto.string_field(1)
    # The event to be published
    event: "NitricEvent" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class EventPublishResponse(betterproto.Message):
    """Result of publishing an event"""

    # The id of the published message When an id was not supplied one should be
    # automatically generated
    id: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class TopicListRequest(betterproto.Message):
    """Request for the Topic List method"""

    pass


@dataclass(eq=False, repr=False)
class TopicListResponse(betterproto.Message):
    """Topic List Response"""

    # The list of found topics
    topics: List["NitricTopic"] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class NitricTopic(betterproto.Message):
    """Represents an event topic"""

    # The Nitric name for the topic
    name: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class NitricEvent(betterproto.Message):
    """Nitric Event Model"""

    # A Unique ID for the Nitric Event
    id: str = betterproto.string_field(1)
    # A content hint for the events payload
    payload_type: str = betterproto.string_field(2)
    # The payload of the event
    payload: "betterproto_lib_google_protobuf.Struct" = betterproto.message_field(3)


class EventServiceStub(betterproto.ServiceStub):
    async def publish(
        self, *, topic: str = "", event: "NitricEvent" = None
    ) -> "EventPublishResponse":

        request = EventPublishRequest()
        request.topic = topic
        if event is not None:
            request.event = event

        return await self._unary_unary(
            "/nitric.event.v1.EventService/Publish", request, EventPublishResponse
        )


class TopicServiceStub(betterproto.ServiceStub):
    async def list(self) -> "TopicListResponse":

        request = TopicListRequest()

        return await self._unary_unary(
            "/nitric.event.v1.TopicService/List", request, TopicListResponse
        )


class EventServiceBase(ServiceBase):
    async def publish(self, topic: str, event: "NitricEvent") -> "EventPublishResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_publish(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "topic": request.topic,
            "event": request.event,
        }

        response = await self.publish(**request_kwargs)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/nitric.event.v1.EventService/Publish": grpclib.const.Handler(
                self.__rpc_publish,
                grpclib.const.Cardinality.UNARY_UNARY,
                EventPublishRequest,
                EventPublishResponse,
            ),
        }


class TopicServiceBase(ServiceBase):
    async def list(self) -> "TopicListResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_list(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {}

        response = await self.list(**request_kwargs)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/nitric.event.v1.TopicService/List": grpclib.const.Handler(
                self.__rpc_list,
                grpclib.const.Cardinality.UNARY_UNARY,
                TopicListRequest,
                TopicListResponse,
            ),
        }


import betterproto.lib.google.protobuf as betterproto_lib_google_protobuf



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: validate/validate.proto
# plugin: python-betterproto
from dataclasses import dataclass
from datetime import datetime, timedelta
from typing import List

import betterproto
from betterproto.grpc.grpclib_server import ServiceBase


class KnownRegex(betterproto.Enum):
    """WellKnownRegex contain some well-known patterns."""

    UNKNOWN = 0
    # HTTP header name as defined by RFC 7230.
    HTTP_HEADER_NAME = 1
    # HTTP header value as defined by RFC 7230.
    HTTP_HEADER_VALUE = 2


@dataclass(eq=False, repr=False)
class FieldRules(betterproto.Message):
    """
    FieldRules encapsulates the rules for each type of field. Depending on the
    field, the correct set should be used to ensure proper validations.
    """

    message: "MessageRules" = betterproto.message_field(17)
    # Scalar Field Types
    float: "FloatRules" = betterproto.message_field(1, group="type")
    double: "DoubleRules" = betterproto.message_field(2, group="type")
    int32: "Int32Rules" = betterproto.message_field(3, group="type")
    int64: "Int64Rules" = betterproto.message_field(4, group="type")
    uint32: "UInt32Rules" = betterproto.message_field(5, group="type")
    uint64: "UInt64Rules" = betterproto.message_field(6, group="type")
    sint32: "SInt32Rules" = betterproto.message_field(7, group="type")
    sint64: "SInt64Rules" = betterproto.message_field(8, group="type")
    fixed32: "Fixed32Rules" = betterproto.message_field(9, group="type")
    fixed64: "Fixed64Rules" = betterproto.message_field(10, group="type")
    sfixed32: "SFixed32Rules" = betterproto.message_field(11, group="type")
    sfixed64: "SFixed64Rules" = betterproto.message_field(12, group="type")
    bool: "BoolRules" = betterproto.message_field(13, group="type")
    string: "StringRules" = betterproto.message_field(14, group="type")
    bytes_: "BytesRules" = betterproto.message_field(15, group="type")
    # Complex Field Types
    enum: "EnumRules" = betterproto.message_field(16, group="type")
    repeated: "RepeatedRules" = betterproto.message_field(18, group="type")
    map: "MapRules" = betterproto.message_field(19, group="type")
    # Well-Known Field Types
    any: "AnyRules" = betterproto.message_field(20, group="type")
    duration: "DurationRules" = betterproto.message_field(21, group="type")
    timestamp: "TimestampRules" = betterproto.message_field(22, group="type")


@dataclass(eq=False, repr=False)
class FloatRules(betterproto.Message):
    """FloatRules describes the constraints applied to `float` values"""

    # Const specifies that this field must be exactly the specified value
    const: float = betterproto.float_field(1)
    # Lt specifies that this field must be less than the specified value,
    # exclusive
    lt: float = betterproto.float_field(2)
    # Lte specifies that this field must be less than or equal to the specified
    # value, inclusive
    lte: float = betterproto.float_field(3)
    # Gt specifies that this field must be greater than the specified value,
    # exclusive. If the value of Gt is larger than a specified Lt or Lte, the
    # range is reversed.
    gt: float = betterproto.float_field(4)
    # Gte specifies that this field must be greater than or equal to the
    # specified value, inclusive. If the value of Gte is larger than a specified
    # Lt or Lte, the range is reversed.
    gte: float = betterproto.float_field(5)
    # In specifies that this field must be equal to one of the specified values
    in_: List[float] = betterproto.float_field(6)
    # NotIn specifies that this field cannot be equal to one of the specified
    # values
    not_in: List[float] = betterproto.float_field(7)
    # IgnoreEmpty specifies that the validation rules of this field should be
    # evaluated only if the field is not empty
    ignore_empty: bool = betterproto.bool_field(8)


@dataclass(eq=False, repr=False)
class DoubleRules(betterproto.Message):
    """DoubleRules describes the constraints applied to `double` values"""

    # Const specifies that this field must be exactly the specified value
    const: float = betterproto.double_field(1)
    # Lt specifies that this field must be less than the specified value,
    # exclusive
    lt: float = betterproto.double_field(2)
    # Lte specifies that this field must be less than or equal to the specified
    # value, inclusive
    lte: float = betterproto.double_field(3)
    # Gt specifies that this field must be greater than the specified value,
    # exclusive. If the value of Gt is larger than a specified Lt or Lte, the
    # range is reversed.
    gt: float = betterproto.double_field(4)
    # Gte specifies that this field must be greater than or equal to the
    # specified value, inclusive. If the value of Gte is larger than a specified
    # Lt or Lte, the range is reversed.
    gte: float = betterproto.double_field(5)
    # In specifies that this field must be equal to one of the specified values
    in_: List[float] = betterproto.double_field(6)
    # NotIn specifies that this field cannot be equal to one of the specified
    # values
    not_in: List[float] = betterproto.double_field(7)
    # IgnoreEmpty specifies that the validation rules of this field should be
    # evaluated only if the field is not empty
    ignore_empty: bool = betterproto.bool_field(8)


@dataclass(eq=False, repr=False)
class Int32Rules(betterproto.Message):
    """Int32Rules describes the constraints applied to `int32` values"""

    # Const specifies that this field must be exactly the specified value
    const: int = betterproto.int32_field(1)
    # Lt specifies that this field must be less than the specified value,
    # exclusive
    lt: int = betterproto.int32_field(2)
    # Lte specifies that this field must be less than or equal to the specified
    # value, inclusive
    lte: int = betterproto.int32_field(3)
    # Gt specifies that this field must be greater than the specified value,
    # exclusive. If the value of Gt is larger than a specified Lt or Lte, the
    # range is reversed.
    gt: int = betterproto.int32_field(4)
    # Gte specifies that this field must be greater than or equal to the
    # specified value, inclusive. If the value of Gte is larger than a specified
    # Lt or Lte, the range is reversed.
    gte: int = betterproto.int32_field(5)
    # In specifies that this field must be equal to one of the specified values
    in_: List[int] = betterproto.int32_field(6)
    # NotIn specifies that this field cannot be equal to one of the specified
    # values
    not_in: List[int] = betterproto.int32_field(7)
    # IgnoreEmpty specifies that the validation rules of this field should be
    # evaluated only if the field is not empty
    ignore_empty: bool = betterproto.bool_field(8)


@dataclass(eq=False, repr=False)
class Int64Rules(betterproto.Message):
    """Int64Rules describes the constraints applied to `int64` values"""

    # Const specifies that this field must be exactly the specified value
    const: int = betterproto.int64_field(1)
    # Lt specifies that this field must be less than the specified value,
    # exclusive
    lt: int = betterproto.int64_field(2)
    # Lte specifies that this field must be less than or equal to the specified
    # value, inclusive
    lte: int = betterproto.int64_field(3)
    # Gt specifies that this field must be greater than the specified value,
    # exclusive. If the value of Gt is larger than a specified Lt or Lte, the
    # range is reversed.
    gt: int = betterproto.int64_field(4)
    # Gte specifies that this field must be greater than or equal to the
    # specified value, inclusive. If the value of Gte is larger than a specified
    # Lt or Lte, the range is reversed.
    gte: int = betterproto.int64_field(5)
    # In specifies that this field must be equal to one of the specified values
    in_: List[int] = betterproto.int64_field(6)
    # NotIn specifies that this field cannot be equal to one of the specified
    # values
    not_in: List[int] = betterproto.int64_field(7)
    # IgnoreEmpty specifies that the validation rules of this field should be
    # evaluated only if the field is not empty
    ignore_empty: bool = betterproto.bool_field(8)


@dataclass(eq=False, repr=False)
class UInt32Rules(betterproto.Message):
    """UInt32Rules describes the constraints applied to `uint32` values"""

    # Const specifies that this field must be exactly the specified value
    const: int = betterproto.uint32_field(1)
    # Lt specifies that this field must be less than the specified value,
    # exclusive
    lt: int = betterproto.uint32_field(2)
    # Lte specifies that this field must be less than or equal to the specified
    # value, inclusive
    lte: int = betterproto.uint32_field(3)
    # Gt specifies that this field must be greater than the specified value,
    # exclusive. If the value of Gt is larger than a specified Lt or Lte, the
    # range is reversed.
    gt: int = betterproto.uint32_field(4)
    # Gte specifies that this field must be greater than or equal to the
    # specified value, inclusive. If the value of Gte is larger than a specified
    # Lt or Lte, the range is reversed.
    gte: int = betterproto.uint32_field(5)
    # In specifies that this field must be equal to one of the specified values
    in_: List[int] = betterproto.uint32_field(6)
    # NotIn specifies that this field cannot be equal to one of the specified
    # values
    not_in: List[int] = betterproto.uint32_field(7)
    # IgnoreEmpty specifies that the validation rules of this field should be
    # evaluated only if the field is not empty
    ignore_empty: bool = betterproto.bool_field(8)


@dataclass(eq=False, repr=False)
class UInt64Rules(betterproto.Message):
    """UInt64Rules describes the constraints applied to `uint64` values"""

    # Const specifies that this field must be exactly the specified value
    const: int = betterproto.uint64_field(1)
    # Lt specifies that this field must be less than the specified value,
    # exclusive
    lt: int = betterproto.uint64_field(2)
    # Lte specifies that this field must be less than or equal to the specified
    # value, inclusive
    lte: int = betterproto.uint64_field(3)
    # Gt specifies that this field must be greater than the specified value,
    # exclusive. If the value of Gt is larger than a specified Lt or Lte, the
    # range is reversed.
    gt: int = betterproto.uint64_field(4)
    # Gte specifies that this field must be greater than or equal to the
    # specified value, inclusive. If the value of Gte is larger than a specified
    # Lt or Lte, the range is reversed.
    gte: int = betterproto.uint64_field(5)
    # In specifies that this field must be equal to one of the specified values
    in_: List[int] = betterproto.uint64_field(6)
    # NotIn specifies that this field cannot be equal to one of the specified
    # values
    not_in: List[int] = betterproto.uint64_field(7)
    # IgnoreEmpty specifies that the validation rules of this field should be
    # evaluated only if the field is not empty
    ignore_empty: bool = betterproto.bool_field(8)


@dataclass(eq=False, repr=False)
class SInt32Rules(betterproto.Message):
    """SInt32Rules describes the constraints applied to `sint32` values"""

    # Const specifies that this field must be exactly the specified value
    const: int = betterproto.sint32_field(1)
    # Lt specifies that this field must be less than the specified value,
    # exclusive
    lt: int = betterproto.sint32_field(2)
    # Lte specifies that this field must be less than or equal to the specified
    # value, inclusive
    lte: int = betterproto.sint32_field(3)
    # Gt specifies that this field must be greater than the specified value,
    # exclusive. If the value of Gt is larger than a specified Lt or Lte, the
    # range is reversed.
    gt: int = betterproto.sint32_field(4)
    # Gte specifies that this field must be greater than or equal to the
    # specified value, inclusive. If the value of Gte is larger than a specified
    # Lt or Lte, the range is reversed.
    gte: int = betterproto.sint32_field(5)
    # In specifies that this field must be equal to one of the specified values
    in_: List[int] = betterproto.sint32_field(6)
    # NotIn specifies that this field cannot be equal to one of the specified
    # values
    not_in: List[int] = betterproto.sint32_field(7)
    # IgnoreEmpty specifies that the validation rules of this field should be
    # evaluated only if the field is not empty
    ignore_empty: bool = betterproto.bool_field(8)


@dataclass(eq=False, repr=False)
class SInt64Rules(betterproto.Message):
    """SInt64Rules describes the constraints applied to `sint64` values"""

    # Const specifies that this field must be exactly the specified value
    const: int = betterproto.sint64_field(1)
    # Lt specifies that this field must be less than the specified value,
    # exclusive
    lt: int = betterproto.sint64_field(2)
    # Lte specifies that this field must be less than or equal to the specified
    # value, inclusive
    lte: int = betterproto.sint64_field(3)
    # Gt specifies that this field must be greater than the specified value,
    # exclusive. If the value of Gt is larger than a specified Lt or Lte, the
    # range is reversed.
    gt: int = betterproto.sint64_field(4)
    # Gte specifies that this field must be greater than or equal to the
    # specified value, inclusive. If the value of Gte is larger than a specified
    # Lt or Lte, the range is reversed.
    gte: int = betterproto.sint64_field(5)
    # In specifies that this field must be equal to one of the specified values
    in_: List[int] = betterproto.sint64_field(6)
    # NotIn specifies that this field cannot be equal to one of the specified
    # values
    not_in: List[int] = betterproto.sint64_field(7)
    # IgnoreEmpty specifies that the validation rules of this field should be
    # evaluated only if the field is not empty
    ignore_empty: bool = betterproto.bool_field(8)


@dataclass(eq=False, repr=False)
class Fixed32Rules(betterproto.Message):
    """Fixed32Rules describes the constraints applied to `fixed32` values"""

    # Const specifies that this field must be exactly the specified value
    const: int = betterproto.fixed32_field(1)
    # Lt specifies that this field must be less than the specified value,
    # exclusive
    lt: int = betterproto.fixed32_field(2)
    # Lte specifies that this field must be less than or equal to the specified
    # value, inclusive
    lte: int = betterproto.fixed32_field(3)
    # Gt specifies that this field must be greater than the specified value,
    # exclusive. If the value of Gt is larger than a specified Lt or Lte, the
    # range is reversed.
    gt: int = betterproto.fixed32_field(4)
    # Gte specifies that this field must be greater than or equal to the
    # specified value, inclusive. If the value of Gte is larger than a specified
    # Lt or Lte, the range is reversed.
    gte: int = betterproto.fixed32_field(5)
    # In specifies that this field must be equal to one of the specified values
    in_: List[int] = betterproto.fixed32_field(6)
    # NotIn specifies that this field cannot be equal to one of the specified
    # values
    not_in: List[int] = betterproto.fixed32_field(7)
    # IgnoreEmpty specifies that the validation rules of this field should be
    # evaluated only if the field is not empty
    ignore_empty: bool = betterproto.bool_field(8)


@dataclass(eq=False, repr=False)
class Fixed64Rules(betterproto.Message):
    """Fixed64Rules describes the constraints applied to `fixed64` values"""

    # Const specifies that this field must be exactly the specified value
    const: int = betterproto.fixed64_field(1)
    # Lt specifies that this field must be less than the specified value,
    # exclusive
    lt: int = betterproto.fixed64_field(2)
    # Lte specifies that this field must be less than or equal to the specified
    # value, inclusive
    lte: int = betterproto.fixed64_field(3)
    # Gt specifies that this field must be greater than the specified value,
    # exclusive. If the value of Gt is larger than a specified Lt or Lte, the
    # range is reversed.
    gt: int = betterproto.fixed64_field(4)
    # Gte specifies that this field must be greater than or equal to the
    # specified value, inclusive. If the value of Gte is larger than a specified
    # Lt or Lte, the range is reversed.
    gte: int = betterproto.fixed64_field(5)
    # In specifies that this field must be equal to one of the specified values
    in_: List[int] = betterproto.fixed64_field(6)
    # NotIn specifies that this field cannot be equal to one of the specified
    # values
    not_in: List[int] = betterproto.fixed64_field(7)
    # IgnoreEmpty specifies that the validation rules of this field should be
    # evaluated only if the field is not empty
    ignore_empty: bool = betterproto.bool_field(8)


@dataclass(eq=False, repr=False)
class SFixed32Rules(betterproto.Message):
    """SFixed32Rules describes the constraints applied to `sfixed32` values"""

    # Const specifies that this field must be exactly the specified value
    const: int = betterproto.sfixed32_field(1)
    # Lt specifies that this field must be less than the specified value,
    # exclusive
    lt: int = betterproto.sfixed32_field(2)
    # Lte specifies that this field must be less than or equal to the specified
    # value, inclusive
    lte: int = betterproto.sfixed32_field(3)
    # Gt specifies that this field must be greater than the specified value,
    # exclusive. If the value of Gt is larger than a specified Lt or Lte, the
    # range is reversed.
    gt: int = betterproto.sfixed32_field(4)
    # Gte specifies that this field must be greater than or equal to the
    # specified value, inclusive. If the value of Gte is larger than a specified
    # Lt or Lte, the range is reversed.
    gte: int = betterproto.sfixed32_field(5)
    # In specifies that this field must be equal to one of the specified values
    in_: List[int] = betterproto.sfixed32_field(6)
    # NotIn specifies that this field cannot be equal to one of the specified
    # values
    not_in: List[int] = betterproto.sfixed32_field(7)
    # IgnoreEmpty specifies that the validation rules of this field should be
    # evaluated only if the field is not empty
    ignore_empty: bool = betterproto.bool_field(8)


@dataclass(eq=False, repr=False)
class SFixed64Rules(betterproto.Message):
    """SFixed64Rules describes the constraints applied to `sfixed64` values"""

    # Const specifies that this field must be exactly the specified value
    const: int = betterproto.sfixed64_field(1)
    # Lt specifies that this field must be less than the specified value,
    # exclusive
    lt: int = betterproto.sfixed64_field(2)
    # Lte specifies that this field must be less than or equal to the specified
    # value, inclusive
    lte: int = betterproto.sfixed64_field(3)
    # Gt specifies that this field must be greater than the specified value,
    # exclusive. If the value of Gt is larger than a specified Lt or Lte, the
    # range is reversed.
    gt: int = betterproto.sfixed64_field(4)
    # Gte specifies that this field must be greater than or equal to the
    # specified value, inclusive. If the value of Gte is larger than a specified
    # Lt or Lte, the range is reversed.
    gte: int = betterproto.sfixed64_field(5)
    # In specifies that this field must be equal to one of the specified values
    in_: List[int] = betterproto.sfixed64_field(6)
    # NotIn specifies that this field cannot be equal to one of the specified
    # values
    not_in: List[int] = betterproto.sfixed64_field(7)
    # IgnoreEmpty specifies that the validation rules of this field should be
    # evaluated only if the field is not empty
    ignore_empty: bool = betterproto.bool_field(8)


@dataclass(eq=False, repr=False)
class BoolRules(betterproto.Message):
    """BoolRules describes the constraints applied to `bool` values"""

    # Const specifies that this field must be exactly the specified value
    const: bool = betterproto.bool_field(1)


@dataclass(eq=False, repr=False)
class StringRules(betterproto.Message):
    """StringRules describe the constraints applied to `string` values"""

    # Const specifies that this field must be exactly the specified value
    const: str = betterproto.string_field(1)
    # Len specifies that this field must be the specified number of characters
    # (Unicode code points). Note that the number of characters may differ from
    # the number of bytes in the string.
    len: int = betterproto.uint64_field(19)
    # MinLen specifies that this field must be the specified number of characters
    # (Unicode code points) at a minimum. Note that the number of characters may
    # differ from the number of bytes in the string.
    min_len: int = betterproto.uint64_field(2)
    # MaxLen specifies that this field must be the specified number of characters
    # (Unicode code points) at a maximum. Note that the number of characters may
    # differ from the number of bytes in the string.
    max_len: int = betterproto.uint64_field(3)
    # LenBytes specifies that this field must be the specified number of bytes at
    # a minimum
    len_bytes: int = betterproto.uint64_field(20)
    # MinBytes specifies that this field must be the specified number of bytes at
    # a minimum
    min_bytes: int = betterproto.uint64_field(4)
    # MaxBytes specifies that this field must be the specified number of bytes at
    # a maximum
    max_bytes: int = betterproto.uint64_field(5)
    # Pattern specifes that this field must match against the specified regular
    # expression (RE2 syntax). The included expression should elide any
    # delimiters.
    pattern: str = betterproto.string_field(6)
    # Prefix specifies that this field must have the specified substring at the
    # beginning of the string.
    prefix: str = betterproto.string_field(7)
    # Suffix specifies that this field must have the specified substring at the
    # end of the string.
    suffix: str = betterproto.string_field(8)
    # Contains specifies that this field must have the specified substring
    # anywhere in the string.
    contains: str = betterproto.string_field(9)
    # NotContains specifies that this field cannot have the specified substring
    # anywhere in the string.
    not_contains: str = betterproto.string_field(23)
    # In specifies that this field must be equal to one of the specified values
    in_: List[str] = betterproto.string_field(10)
    # NotIn specifies that this field cannot be equal to one of the specified
    # values
    not_in: List[str] = betterproto.string_field(11)
    # Email specifies that the field must be a valid email address as defined by
    # RFC 5322
    email: bool = betterproto.bool_field(12, group="well_known")
    # Hostname specifies that the field must be a valid hostname as defined by
    # RFC 1034. This constraint does not support internationalized domain names
    # (IDNs).
    hostname: bool = betterproto.bool_field(13, group="well_known")
    # Ip specifies that the field must be a valid IP (v4 or v6) address. Valid
    # IPv6 addresses should not include surrounding square brackets.
    ip: bool = betterproto.bool_field(14, group="well_known")
    # Ipv4 specifies that the field must be a valid IPv4 address.
    ipv4: bool = betterproto.bool_field(15, group="well_known")
    # Ipv6 specifies that the field must be a valid IPv6 address. Valid IPv6
    # addresses should not include surrounding square brackets.
    ipv6: bool = betterproto.bool_field(16, group="well_known")
    # Uri specifies that the field must be a valid, absolute URI as defined by
    # RFC 3986
    uri: bool = betterproto.bool_field(17, group="well_known")
    # UriRef specifies that the field must be a valid URI as defined by RFC 3986
    # and may be relative or absolute.
    uri_ref: bool = betterproto.bool_field(18, group="well_known")
    # Address specifies that the field must be either a valid hostname as defined
    # by RFC 1034 (which does not support internationalized domain names or
    # IDNs), or it can be a valid IP (v4 or v6).
    address: bool = betterproto.bool_field(21, group="well_known")
    # Uuid specifies that the field must be a valid UUID as defined by RFC 4122
    uuid: bool = betterproto.bool_field(22, group="well_known")
    # WellKnownRegex specifies a common well known pattern defined as a regex.
    well_known_regex: "KnownRegex" = betterproto.enum_field(24, group="well_known")
    # This applies to regexes HTTP_HEADER_NAME and HTTP_HEADER_VALUE to enable
    # strict header validation. By default, this is true, and HTTP header
    # validations are RFC-compliant. Setting to false will enable a looser
    # validations that only disallows \r\n\0 characters, which can be used to
    # bypass header matching rules.
    strict: bool = betterproto.bool_field(25)
    # IgnoreEmpty specifies that the validation rules of this field should be
    # evaluated only if the field is not empty
    ignore_empty: bool = betterproto.bool_field(26)


@dataclass(eq=False, repr=False)
class BytesRules(betterproto.Message):
    """BytesRules describe the constraints applied to `bytes` values"""

    # Const specifies that this field must be exactly the specified value
    const: bytes = betterproto.bytes_field(1)
    # Len specifies that this field must be the specified number of bytes
    len: int = betterproto.uint64_field(13)
    # MinLen specifies that this field must be the specified number of bytes at a
    # minimum
    min_len: int = betterproto.uint64_field(2)
    # MaxLen specifies that this field must be the specified number of bytes at a
    # maximum
    max_len: int = betterproto.uint64_field(3)
    # Pattern specifes that this field must match against the specified regular
    # expression (RE2 syntax). The included expression should elide any
    # delimiters.
    pattern: str = betterproto.string_field(4)
    # Prefix specifies that this field must have the specified bytes at the
    # beginning of the string.
    prefix: bytes = betterproto.bytes_field(5)
    # Suffix specifies that this field must have the specified bytes at the end
    # of the string.
    suffix: bytes = betterproto.bytes_field(6)
    # Contains specifies that this field must have the specified bytes anywhere
    # in the string.
    contains: bytes = betterproto.bytes_field(7)
    # In specifies that this field must be equal to one of the specified values
    in_: List[bytes] = betterproto.bytes_field(8)
    # NotIn specifies that this field cannot be equal to one of the specified
    # values
    not_in: List[bytes] = betterproto.bytes_field(9)
    # Ip specifies that the field must be a valid IP (v4 or v6) address in byte
    # format
    ip: bool = betterproto.bool_field(10, group="well_known")
    # Ipv4 specifies that the field must be a valid IPv4 address in byte format
    ipv4: bool = betterproto.bool_field(11, group="well_known")
    # Ipv6 specifies that the field must be a valid IPv6 address in byte format
    ipv6: bool = betterproto.bool_field(12, group="well_known")
    # IgnoreEmpty specifies that the validation rules of this field should be
    # evaluated only if the field is not empty
    ignore_empty: bool = betterproto.bool_field(14)


@dataclass(eq=False, repr=False)
class EnumRules(betterproto.Message):
    """EnumRules describe the constraints applied to enum values"""

    # Const specifies that this field must be exactly the specified value
    const: int = betterproto.int32_field(1)
    # DefinedOnly specifies that this field must be only one of the defined
    # values for this enum, failing on any undefined value.
    defined_only: bool = betterproto.bool_field(2)
    # In specifies that this field must be equal to one of the specified values
    in_: List[int] = betterproto.int32_field(3)
    # NotIn specifies that this field cannot be equal to one of the specified
    # values
    not_in: List[int] = betterproto.int32_field(4)


@dataclass(eq=False, repr=False)
class MessageRules(betterproto.Message):
    """
    MessageRules describe the constraints applied to embedded message values.
    For message-type fields, validation is performed recursively.
    """

    # Skip specifies that the validation rules of this field should not be
    # evaluated
    skip: bool = betterproto.bool_field(1)
    # Required specifies that this field must be set
    required: bool = betterproto.bool_field(2)


@dataclass(eq=False, repr=False)
class RepeatedRules(betterproto.Message):
    """RepeatedRules describe the constraints applied to `repeated` values"""

    # MinItems specifies that this field must have the specified number of items
    # at a minimum
    min_items: int = betterproto.uint64_field(1)
    # MaxItems specifies that this field must have the specified number of items
    # at a maximum
    max_items: int = betterproto.uint64_field(2)
    # Unique specifies that all elements in this field must be unique. This
    # contraint is only applicable to scalar and enum types (messages are not
    # supported).
    unique: bool = betterproto.bool_field(3)
    # Items specifies the contraints to be applied to each item in the field.
    # Repeated message fields will still execute validation against each item
    # unless skip is specified here.
    items: "FieldRules" = betterproto.message_field(4)
    # IgnoreEmpty specifies that the validation rules of this field should be
    # evaluated only if the field is not empty
    ignore_empty: bool = betterproto.bool_field(5)


@dataclass(eq=False, repr=False)
class MapRules(betterproto.Message):
    """MapRules describe the constraints applied to `map` values"""

    # MinPairs specifies that this field must have the specified number of KVs at
    # a minimum
    min_pairs: int = betterproto.uint64_field(1)
    # MaxPairs specifies that this field must have the specified number of KVs at
    # a maximum
    max_pairs: int = betterproto.uint64_field(2)
    # NoSparse specifies values in this field cannot be unset. This only applies
    # to map's with message value types.
    no_sparse: bool = betterproto.bool_field(3)
    # Keys specifies the constraints to be applied to each key in the field.
    keys: "FieldRules" = betterproto.message_field(4)
    # Values specifies the constraints to be applied to the value of each key in
    # the field. Message values will still have their validations evaluated
    # unless skip is specified here.
    values: "FieldRules" = betterproto.message_field(5)
    # IgnoreEmpty specifies that the validation rules of this field should be
    # evaluated only if the field is not empty
    ignore_empty: bool = betterproto.bool_field(6)


@dataclass(eq=False, repr=False)
class AnyRules(betterproto.Message):
    """
    AnyRules describe constraints applied exclusively to the
    `google.protobuf.Any` well-known type
    """

    # Required specifies that this field must be set
    required: bool = betterproto.bool_field(1)
    # In specifies that this field's `type_url` must be equal to one of the
    # specified values.
    in_: List[str] = betterproto.string_field(2)
    # NotIn specifies that this field's `type_url` must not be equal to any of
    # the specified values.
    not_in: List[str] = betterproto.string_field(3)


@dataclass(eq=False, repr=False)
class DurationRules(betterproto.Message):
    """
    DurationRules describe the constraints applied exclusively to the
    `google.protobuf.Duration` well-known type
    """

    # Required specifies that this field must be set
    required: bool = betterproto.bool_field(1)
    # Const specifies that this field must be exactly the specified value
    const: timedelta = betterproto.message_field(2)
    # Lt specifies that this field must be less than the specified value,
    # exclusive
    lt: timedelta = betterproto.message_field(3)
    # Lt specifies that this field must be less than the specified value,
    # inclusive
    lte: timedelta = betterproto.message_field(4)
    # Gt specifies that this field must be greater than the specified value,
    # exclusive
    gt: timedelta = betterproto.message_field(5)
    # Gte specifies that this field must be greater than the specified value,
    # inclusive
    gte: timedelta = betterproto.message_field(6)
    # In specifies that this field must be equal to one of the specified values
    in_: List[timedelta] = betterproto.message_field(7)
    # NotIn specifies that this field cannot be equal to one of the specified
    # values
    not_in: List[timedelta] = betterproto.message_field(8)


@dataclass(eq=False, repr=False)
class TimestampRules(betterproto.Message):
    """
    TimestampRules describe the constraints applied exclusively to the
    `google.protobuf.Timestamp` well-known type
    """

    # Required specifies that this field must be set
    required: bool = betterproto.bool_field(1)
    # Const specifies that this field must be exactly the specified value
    const: datetime = betterproto.message_field(2)
    # Lt specifies that this field must be less than the specified value,
    # exclusive
    lt: datetime = betterproto.message_field(3)
    # Lte specifies that this field must be less than the specified value,
    # inclusive
    lte: datetime = betterproto.message_field(4)
    # Gt specifies that this field must be greater than the specified value,
    # exclusive
    gt: datetime = betterproto.message_field(5)
    # Gte specifies that this field must be greater than the specified value,
    # inclusive
    gte: datetime = betterproto.message_field(6)
    # LtNow specifies that this must be less than the current time. LtNow can
    # only be used with the Within rule.
    lt_now: bool = betterproto.bool_field(7)
    # GtNow specifies that this must be greater than the current time. GtNow can
    # only be used with the Within rule.
    gt_now: bool = betterproto.bool_field(8)
    # Within specifies that this field must be within this duration of the
    # current time. This constraint can be used alone or with the LtNow and GtNow
    # rules.
    within: timedelta = betterproto.message_field(9)



import setuptools
import re
from subprocess import Popen, PIPE


def get_current_version_tag():
    # Get tags on the current checked out commit only
    process = Popen(["git", "describe", "--tags", "HEAD"], stdout=PIPE)
    (output, err) = process.communicate()
    process.wait()

    tags = str(output, "utf-8").strip().split("\n")

    if len(tags) == 1:
        return tags.pop()[1:]
    else:
        # throw error
        raise Exception("Could not determine version")


with open("README.md", "r") as readme_file:
    long_description = readme_file.read()

setuptools.setup(
    name="nitric-api",
    version=get_current_version_tag(),
    author="Nitric",
    author_email="maintainers@nitric.io",
    description="The Nitric base SDK for Python 3",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/nitrictech/apis",
    packages=setuptools.find_packages(exclude=["tests", "tests.*"]),
    license_files=("LICENSE.txt",),
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    setup_requires=["wheel"],
    install_requires=[
        "protobuf==3.19.4",
        "betterproto==2.0.0b3",
        "asyncio",
    ],
    extras_require={
        "dev": [
            # needed to lock markupsafe to v2.0.1 to avoid build errors.
            # see: https://github.com/danielgtaylor/python-betterproto/issues/341
            "markupsafe==2.0.1",
            "betterproto[compiler]==2.0.0b3",
            # "grpcio==1.33.2",
            "grpcio-tools==1.44.0",
            "twine==3.2.0",
        ]
    },
    python_requires=">=3.7",
)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: proto/secret/v1/secret.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import Dict

import betterproto
from betterproto.grpc.grpclib_server import ServiceBase
import grpclib


@dataclass(eq=False, repr=False)
class SecretPutRequest(betterproto.Message):
    """Request to put a secret to a Secret Store"""

    # The Secret to put to the Secret store
    secret: "Secret" = betterproto.message_field(1)
    # The value to assign to that secret
    value: bytes = betterproto.bytes_field(2)


@dataclass(eq=False, repr=False)
class SecretPutResponse(betterproto.Message):
    """Result from putting the secret to a Secret Store"""

    # The id of the secret
    secret_version: "SecretVersion" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class SecretAccessRequest(betterproto.Message):
    """Request to get a secret from a Secret Store"""

    # The id of the secret
    secret_version: "SecretVersion" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class SecretAccessResponse(betterproto.Message):
    """The secret response"""

    # The version of the secret that was requested
    secret_version: "SecretVersion" = betterproto.message_field(1)
    # The value of the secret
    value: bytes = betterproto.bytes_field(2)


@dataclass(eq=False, repr=False)
class Secret(betterproto.Message):
    """The secret container"""

    # The secret name
    name: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class SecretVersion(betterproto.Message):
    """A version of a secret"""

    # Reference to the secret container
    secret: "Secret" = betterproto.message_field(1)
    # The secret version
    version: str = betterproto.string_field(2)


class SecretServiceStub(betterproto.ServiceStub):
    async def put(
        self, *, secret: "Secret" = None, value: bytes = b""
    ) -> "SecretPutResponse":

        request = SecretPutRequest()
        if secret is not None:
            request.secret = secret
        request.value = value

        return await self._unary_unary(
            "/nitric.secret.v1.SecretService/Put", request, SecretPutResponse
        )

    async def access(
        self, *, secret_version: "SecretVersion" = None
    ) -> "SecretAccessResponse":

        request = SecretAccessRequest()
        if secret_version is not None:
            request.secret_version = secret_version

        return await self._unary_unary(
            "/nitric.secret.v1.SecretService/Access", request, SecretAccessResponse
        )


class SecretServiceBase(ServiceBase):
    async def put(self, secret: "Secret", value: bytes) -> "SecretPutResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def access(self, secret_version: "SecretVersion") -> "SecretAccessResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_put(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "secret": request.secret,
            "value": request.value,
        }

        response = await self.put(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_access(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "secret_version": request.secret_version,
        }

        response = await self.access(**request_kwargs)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/nitric.secret.v1.SecretService/Put": grpclib.const.Handler(
                self.__rpc_put,
                grpclib.const.Cardinality.UNARY_UNARY,
                SecretPutRequest,
                SecretPutResponse,
            ),
            "/nitric.secret.v1.SecretService/Access": grpclib.const.Handler(
                self.__rpc_access,
                grpclib.const.Cardinality.UNARY_UNARY,
                SecretAccessRequest,
                SecretAccessResponse,
            ),
        }



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: proto/document/v1/document.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import AsyncIterator, Dict, List, Optional

import betterproto
from betterproto.grpc.grpclib_server import ServiceBase
import grpclib


@dataclass(eq=False, repr=False)
class Collection(betterproto.Message):
    """Provides a Collection type for storing documents"""

    # The collection name
    name: str = betterproto.string_field(1)
    # Optional parent key, required when the collection is a sub-collection of
    # another document
    parent: "Key" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class Key(betterproto.Message):
    """Provides a document identifying key type"""

    # The item collection
    collection: "Collection" = betterproto.message_field(1)
    # The items unique id
    id: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class Document(betterproto.Message):
    """Provides a return document type"""

    # The document content (JSON object)
    content: "betterproto_lib_google_protobuf.Struct" = betterproto.message_field(1)
    # The document's unique key, including collection/sub-collections
    key: "Key" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class ExpressionValue(betterproto.Message):
    # Represents an integer value.
    int_value: int = betterproto.int64_field(1, group="kind")
    # Represents a double value.
    double_value: float = betterproto.double_field(2, group="kind")
    # Represents a string value.
    string_value: str = betterproto.string_field(3, group="kind")
    # Represents a boolean value.
    bool_value: bool = betterproto.bool_field(4, group="kind")


@dataclass(eq=False, repr=False)
class Expression(betterproto.Message):
    """Provides a query expression type"""

    # The query operand or attribute
    operand: str = betterproto.string_field(1)
    # The query operator [ == | < | <= | > | >= | startsWith ]
    operator: str = betterproto.string_field(2)
    # The query expression value
    value: "ExpressionValue" = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class DocumentGetRequest(betterproto.Message):
    # Key of the document to retrieve
    key: "Key" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class DocumentGetResponse(betterproto.Message):
    # The retrieved value
    document: "Document" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class DocumentSetRequest(betterproto.Message):
    # Key of the document to set
    key: "Key" = betterproto.message_field(1)
    # The document content to store (JSON object)
    content: "betterproto_lib_google_protobuf.Struct" = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class DocumentSetResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class DocumentDeleteRequest(betterproto.Message):
    # Key of the document to delete
    key: "Key" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class DocumentDeleteResponse(betterproto.Message):
    pass


@dataclass(eq=False, repr=False)
class DocumentQueryRequest(betterproto.Message):
    # The collection to query
    collection: "Collection" = betterproto.message_field(1)
    # Optional query expressions
    expressions: List["Expression"] = betterproto.message_field(3)
    # Optional query fetch limit
    limit: int = betterproto.int32_field(4)
    # Optional query paging continuation token
    paging_token: Dict[str, str] = betterproto.map_field(
        5, betterproto.TYPE_STRING, betterproto.TYPE_STRING
    )


@dataclass(eq=False, repr=False)
class DocumentQueryResponse(betterproto.Message):
    # The retrieved values
    documents: List["Document"] = betterproto.message_field(1)
    # The query paging continuation token, when empty no further results are
    # available
    paging_token: Dict[str, str] = betterproto.map_field(
        2, betterproto.TYPE_STRING, betterproto.TYPE_STRING
    )


@dataclass(eq=False, repr=False)
class DocumentQueryStreamRequest(betterproto.Message):
    # The collection to query
    collection: "Collection" = betterproto.message_field(1)
    # Optional query expressions
    expressions: List["Expression"] = betterproto.message_field(3)
    # Optional query fetch limit
    limit: int = betterproto.int32_field(4)


@dataclass(eq=False, repr=False)
class DocumentQueryStreamResponse(betterproto.Message):
    # The stream document
    document: "Document" = betterproto.message_field(1)


class DocumentServiceStub(betterproto.ServiceStub):
    async def get(self, *, key: "Key" = None) -> "DocumentGetResponse":

        request = DocumentGetRequest()
        if key is not None:
            request.key = key

        return await self._unary_unary(
            "/nitric.document.v1.DocumentService/Get", request, DocumentGetResponse
        )

    async def set(
        self,
        *,
        key: "Key" = None,
        content: "betterproto_lib_google_protobuf.Struct" = None,
    ) -> "DocumentSetResponse":

        request = DocumentSetRequest()
        if key is not None:
            request.key = key
        if content is not None:
            request.content = content

        return await self._unary_unary(
            "/nitric.document.v1.DocumentService/Set", request, DocumentSetResponse
        )

    async def delete(self, *, key: "Key" = None) -> "DocumentDeleteResponse":

        request = DocumentDeleteRequest()
        if key is not None:
            request.key = key

        return await self._unary_unary(
            "/nitric.document.v1.DocumentService/Delete",
            request,
            DocumentDeleteResponse,
        )

    async def query(
        self,
        *,
        collection: "Collection" = None,
        expressions: Optional[List["Expression"]] = None,
        limit: int = 0,
        paging_token: Dict[str, str] = None,
    ) -> "DocumentQueryResponse":
        expressions = expressions or []

        request = DocumentQueryRequest()
        if collection is not None:
            request.collection = collection
        if expressions is not None:
            request.expressions = expressions
        request.limit = limit
        request.paging_token = paging_token

        return await self._unary_unary(
            "/nitric.document.v1.DocumentService/Query", request, DocumentQueryResponse
        )

    async def query_stream(
        self,
        *,
        collection: "Collection" = None,
        expressions: Optional[List["Expression"]] = None,
        limit: int = 0,
    ) -> AsyncIterator["DocumentQueryStreamResponse"]:
        expressions = expressions or []

        request = DocumentQueryStreamRequest()
        if collection is not None:
            request.collection = collection
        if expressions is not None:
            request.expressions = expressions
        request.limit = limit

        async for response in self._unary_stream(
            "/nitric.document.v1.DocumentService/QueryStream",
            request,
            DocumentQueryStreamResponse,
        ):
            yield response


class DocumentServiceBase(ServiceBase):
    async def get(self, key: "Key") -> "DocumentGetResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def set(
        self, key: "Key", content: "betterproto_lib_google_protobuf.Struct"
    ) -> "DocumentSetResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def delete(self, key: "Key") -> "DocumentDeleteResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def query(
        self,
        collection: "Collection",
        expressions: Optional[List["Expression"]],
        limit: int,
        paging_token: Dict[str, str],
    ) -> "DocumentQueryResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def query_stream(
        self,
        collection: "Collection",
        expressions: Optional[List["Expression"]],
        limit: int,
    ) -> AsyncIterator["DocumentQueryStreamResponse"]:
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_get(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "key": request.key,
        }

        response = await self.get(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_set(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "key": request.key,
            "content": request.content,
        }

        response = await self.set(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_delete(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "key": request.key,
        }

        response = await self.delete(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_query(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "collection": request.collection,
            "expressions": request.expressions,
            "limit": request.limit,
            "paging_token": request.paging_token,
        }

        response = await self.query(**request_kwargs)
        await stream.send_message(response)

    async def __rpc_query_stream(self, stream: grpclib.server.Stream) -> None:
        request = await stream.recv_message()

        request_kwargs = {
            "collection": request.collection,
            "expressions": request.expressions,
            "limit": request.limit,
        }

        await self._call_rpc_handler_server_stream(
            self.query_stream,
            stream,
            request_kwargs,
        )

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/nitric.document.v1.DocumentService/Get": grpclib.const.Handler(
                self.__rpc_get,
                grpclib.const.Cardinality.UNARY_UNARY,
                DocumentGetRequest,
                DocumentGetResponse,
            ),
            "/nitric.document.v1.DocumentService/Set": grpclib.const.Handler(
                self.__rpc_set,
                grpclib.const.Cardinality.UNARY_UNARY,
                DocumentSetRequest,
                DocumentSetResponse,
            ),
            "/nitric.document.v1.DocumentService/Delete": grpclib.const.Handler(
                self.__rpc_delete,
                grpclib.const.Cardinality.UNARY_UNARY,
                DocumentDeleteRequest,
                DocumentDeleteResponse,
            ),
            "/nitric.document.v1.DocumentService/Query": grpclib.const.Handler(
                self.__rpc_query,
                grpclib.const.Cardinality.UNARY_UNARY,
                DocumentQueryRequest,
                DocumentQueryResponse,
            ),
            "/nitric.document.v1.DocumentService/QueryStream": grpclib.const.Handler(
                self.__rpc_query_stream,
                grpclib.const.Cardinality.UNARY_STREAM,
                DocumentQueryStreamRequest,
                DocumentQueryStreamResponse,
            ),
        }


import betterproto.lib.google.protobuf as betterproto_lib_google_protobuf
